@@ -1,4 +1,4 @@
-#[allow(unused)]
+extern crate threadpool;
 mod libs;
 pub mod server;
 pub mod structs;
