@@ -1,8 +1,9 @@
+use crate::libs::found_callback::{found_callback, IsFound};
 use crate::libs::http_status_string::http_status_string;
 use crate::libs::parse::parse;
-use crate::server::MiddlewareCallback;
 use crate::server::Server;
-use crate::structs::{Context, Request, Response};
+use crate::structs::{Context, MiddlewareCallback, Request, Response, RouteCallback};
+use futures::future::join_all;
 use std::borrow::Cow;
 use std::collections::HashMap;
 use std::io::{Read, Write};
@@ -11,20 +12,23 @@ use std::net::TcpStream;
 /*
  * Handler
  */
-pub(crate) fn handler(mut stream: TcpStream, server: Server) -> () {
-    let client_ip: String = stream.peer_addr().unwrap().to_string();
-    let client_ip: Vec<String> = client_ip.split(":").map(|s| s.to_string()).collect();
+pub(crate) async fn handler(mut stream: TcpStream, server: Server) -> () {
     /*
      * Buffer
      */
-    let mut buffer: [u8; 1024 * 8] = [0; 1024 * 8];
+    let mut buffer: [u8; 1024] = [0; 1024];
     stream.read(&mut buffer).unwrap();
     /*
      * Request Header
      */
-    let header: Cow<str> = String::from_utf8_lossy(&buffer);
-    let mut header: HashMap<String, String> = parse(header.to_string());
-    header.insert("ip".to_string(), client_ip[0].to_string());
+    let header: Cow<str> = String::from_utf8_lossy(&buffer[..]);
+    let mut header: HashMap<String, String> = parse(header.to_string()).await;
+    /*
+     * Client IP
+     */
+    let client_ip: String = stream.peer_addr().unwrap().to_string();
+    let client_ip: String = client_ip.split(":").next().unwrap_or("").to_string();
+    header.insert("ip".to_string(), client_ip);
     /*
      * Response Header Default
      */
@@ -49,8 +53,6 @@ pub(crate) fn handler(mut stream: TcpStream, server: Server) -> () {
      * Incoming Path
      */
     let path: String = header.get("path").unwrap().to_string().to_lowercase();
-    let mut path_split: Vec<String> = path.clone().split("/").map(|s| s.to_string()).collect();
-    path_split.remove(0);
     /*
      * Next Execution
      */
@@ -59,13 +61,17 @@ pub(crate) fn handler(mut stream: TcpStream, server: Server) -> () {
      * Middlewares
      */
     let r: Vec<MiddlewareCallback> = server.middlewares;
+
     let mut middleware_ends: Vec<Box<dyn Fn(&mut Context) -> ()>> = Vec::new();
+
     for i in r {
         let (next, next_callback) = (i)(&mut context);
+
         if !next {
             next_exec = false;
             break;
         }
+
         if next_callback.is_some() {
             middleware_ends.push(next_callback.unwrap());
         }
@@ -75,98 +81,35 @@ pub(crate) fn handler(mut stream: TcpStream, server: Server) -> () {
         /*
          * Routes
          */
-        let mut r: Vec<(String, fn(&mut Context) -> ())> = Vec::new();
-
-        if header.get("method").unwrap() == "GET" {
-            r = server.gets;
-        } else if header.get("method").unwrap() == "POST" {
-            r = server.posts;
-        } else if header.get("method").unwrap() == "PUT" {
-            r = server.puts;
-        } else if header.get("method").unwrap() == "DELETE" {
-            r = server.deletes;
-        } else if header.get("method").unwrap() == "PATCH" {
-            r = server.patchs;
-        }
+        let r: Vec<(String, RouteCallback)> = match header.get("method").unwrap().as_str() {
+            "GET" => server.gets,
+            "POST" => server.posts,
+            "PUT" => server.puts,
+            "DELETE" => server.deletes,
+            "PATCH" => server.patchs,
+            _ => Vec::new(),
+        };
 
         let mut is_found: bool = false;
+        /*
+         * Find & Call Route Callback
+         */
+        let found_callback_iter = r
+            .iter()
+            .map(|i| found_callback(path.clone(), i.0.to_lowercase()));
 
-        for i in r {
-            /*
-             * Current Path
-             */
-            let path_curr: String = i.0.to_lowercase();
-            /*
-             * Static Match
-             */
-            if path_curr == path {
-                (i.1)(&mut context);
-                is_found = true;
-                break;
-            }
-            /*
-             * Dynamic Match
-             */
-            let mut path_curr_split: Vec<String> = path_curr
-                .clone()
-                .split("/")
-                .map(|s| s.to_string())
-                .collect();
+        let found_callback_iter: Vec<IsFound> = join_all(found_callback_iter).await;
 
-            let mut path_curr_split_filter: Vec<String> = Vec::new();
-            for p in path_curr_split {
-                if p.len() > 0 {
-                    path_curr_split_filter.push(p);
-                }
-            }
-            /*
-             * Check Split Length
-             */
-            if path_curr_split_filter.len() != path_split.len() {
-                continue;
-            }
-
-            let mut prepare_path: String = String::from("");
-            /*
-             * Check Params
-             */
-            for j in 0..path_curr_split_filter.len() {
-                let path_elm = path_curr_split_filter[j].clone();
-                /*
-                 * First Element
-                 */
-                if path_elm.len() < 1 {
-                    continue;
-                }
-
-                let path_elm_char: String = path_elm.clone().chars().nth(0).unwrap().to_string();
-
-                /*
-                 * Dynamic Param
-                 */
-                if path_elm_char == ":" {
-                    prepare_path.push_str(&format!("/{}", path_elm));
-                    context
-                        .request
-                        .params
-                        .insert(path_elm.replace(":", ""), path_split[j].to_string());
-                }
-                /*
-                 * Static Param
-                 */
-                else if path_elm == path_split[j] {
-                    prepare_path.push_str(&format!("/{}", path_elm));
-                }
-            }
-            /*
-             * Match current path with prepare path
-             */
-            if path_curr == prepare_path {
-                (i.1)(&mut context);
+        for (i, el) in r.iter().enumerate() {
+            if found_callback_iter[i].found && found_callback_iter[i].path == el.0.to_lowercase() {
+                context.request.params = found_callback_iter[i].params.clone();
+                (el.1)(&mut context);
                 is_found = true;
                 break;
             }
         }
+
+        drop(found_callback_iter);
         /*
          * Error
          */
@@ -181,13 +124,9 @@ pub(crate) fn handler(mut stream: TcpStream, server: Server) -> () {
             /*
              * Check Catch Callback Exists
              */
-            let catch_route: bool = match server.catchs {
-                None => false,
-                _ => true,
-            };
-
-            if catch_route {
-                (server.catchs).unwrap()(&mut context);
+            if server.catchs.is_some() {
+                let cb = server.catchs.unwrap();
+                (cb)(&mut context);
             } else {
                 /*
                  * Default Body
@@ -198,20 +137,18 @@ pub(crate) fn handler(mut stream: TcpStream, server: Server) -> () {
         /*
          * Middlewares End
          */
-        for i in middleware_ends.iter().rev() {
-            (i)(&mut context);
-        }
+        middleware_ends.iter().rev().for_each(|i| (i)(&mut context));
     }
     /*
      * Prepare Response Headers
      */
     let mut response_header: String = String::from("");
-    for (k, v) in &context.response.header {
+    context.response.header.iter().for_each(|(k, v)| {
         if k == "status" || k == "type" {
-            continue;
+            return;
         }
         response_header.push_str(&format!("{}: {}\r\n", k, v));
-    }
+    });
     /*
      * Prepare Response Payload
      */
