@@ -1,19 +1,16 @@
 use std::collections::HashMap;
 
-pub(crate) fn parse(str: String) -> HashMap<String, String> {
-    let h: Vec<&str> = str.lines().collect();
+pub(crate) async fn parse(str: String) -> HashMap<String, String> {
+    let mut h: Vec<&str> = str.lines().collect();
     let h_firstln: Vec<String> = h[0].split_whitespace().map(|s| s.to_string()).collect();
+    h.remove(0);
     /*
      * Split path with Path & Query
      */
     let mut path_split: Vec<String> = vec!["/".to_string()];
 
     if h_firstln.get(1).is_some() {
-        path_split = h_firstln[1]
-            .clone()
-            .split("?")
-            .map(|s| s.to_string())
-            .collect();
+        path_split = h_firstln[1].split("?").map(|s| s.to_string()).collect();
     }
     /*
      * Separate Path & Query
@@ -24,13 +21,17 @@ pub(crate) fn parse(str: String) -> HashMap<String, String> {
     if path_split.get(0).is_some() {
         path = path_split[0].clone();
     }
+
     if path_split.get(1).is_some() {
         query = path_split[1].clone();
     }
+
+    drop(path_split);
     /*
      * Http Version
      */
     let mut http_version: String = "".to_string();
+
     if h_firstln.get(2).is_some() {
         http_version = match h_firstln[2].as_str() {
             "HTTP/3.0" => "3.0".to_string(),
@@ -49,11 +50,12 @@ pub(crate) fn parse(str: String) -> HashMap<String, String> {
     header.insert("query".to_string(), query);
     header.insert("http-version".to_string(), http_version);
 
-    for i in 1..h.len() {
-        let ln = h[i];
+    drop(h_firstln);
+
+    h.iter().for_each(|ln| {
         let mut ln_split: Vec<String> = ln.split_whitespace().map(|s| s.to_string()).collect();
         if ln_split.len() < 1 {
-            continue;
+            return;
         }
         /*
          * Key
@@ -69,10 +71,8 @@ pub(crate) fn parse(str: String) -> HashMap<String, String> {
          */
         let k: Vec<String> = k.split_whitespace().map(|s| s.to_string()).collect();
         let k: String = k.join("");
-        let k: Vec<String> = k.lines().map(|s| s.to_string()).collect();
-        let k: String = k.join("");
         if k.len() < 1 {
-            continue;
+            return;
         }
         /*
          * Value
@@ -82,15 +82,15 @@ pub(crate) fn parse(str: String) -> HashMap<String, String> {
         if ln_split.get(1).is_some() {
             v = ln_split[1].clone().to_string();
             if ln_split.len() > 2 {
-                let _ = ln_split.remove(0);
+                ln_split.remove(0);
                 v = ln_split.join(" ").to_string();
             }
         }
 
         header.insert(k, v);
-    }
-    /*
-     * Return Parsed Data
-     */
+    });
+
+    drop(h);
+
     header
 }
