@@ -3,6 +3,7 @@ use std::collections::HashMap;
 /*
  * Request
  */
+#[derive(Clone, Debug)]
 pub struct Request {
     /// Get Request Haders
     ///
@@ -71,11 +72,11 @@ impl Request {
 
         let query_split: Vec<String> = query_hstr.split("&").map(|s| s.to_string()).collect();
 
-        for q in query_split {
+        query_split.iter().for_each(|q| {
             let mut kv: Vec<String> = q.split("=").map(|s| s.to_string()).collect();
 
             if kv.get(0).is_none() || kv[0].len() < 1 {
-                continue;
+                return;
             }
 
             let k: String = kv[0].clone().to_lowercase();
@@ -84,13 +85,13 @@ impl Request {
             if kv.get(1).is_some() {
                 v = kv[1].clone();
                 if kv.len() > 2 {
-                    let _ = kv.remove(0);
+                    kv.remove(0);
                     v = kv.join("=").to_string();
                 }
             }
 
             query_str.insert(k, v);
-        }
+        });
 
         query_str
     }
@@ -98,6 +99,7 @@ impl Request {
 /*
  * Response
  */
+#[derive(Clone, Debug)]
 pub struct Response {
     /// Set, Change & Remove Response Headers
     ///
@@ -126,11 +128,11 @@ pub struct Response {
     /// use oxidy::server::Server;
     ///
     /// fn index (ctx: &mut Context) -> () {
-    ///     # To Get Body
-    ///     let body = ctx.response.body;
+    ///     /* To Get Body */
+    ///     let body = &ctx.response.body;
     ///     println!("{}", body);
     ///
-    ///     # To Set Body
+    ///     /* To Set Body */
     ///     ctx.response.body = "<h1>Hello World</h1>".to_string();
     /// }
     ///
@@ -143,6 +145,7 @@ pub struct Response {
 /*
  * Context
  */
+#[derive(Clone, Debug)]
 pub struct Context {
     pub request: Request,
     pub response: Response,
@@ -163,6 +166,7 @@ pub struct Context {
     /// }
     ///
     /// let mut app = Server::new();
+    /// app.middleware(mid);
     /// let a = app.get("/", index);
     /// assert_eq!((), a);
     /// ```
@@ -172,3 +176,8 @@ pub struct Context {
  * Middleware Callback Return Type
  */
 pub type Middleware = (bool, Option<Box<dyn Fn(&mut Context) -> ()>>);
+pub(crate) type MiddlewareCallback = fn(&mut Context) -> Middleware;
+/*
+ * Route Callback Return Type
+ */
+pub(crate) type RouteCallback = fn(&mut Context) -> ();
